﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Windows.Forms.AxHost;

namespace Bulldozer
{
    /// <summary>
    ///  Класс отрисовки и перемещения объекта-сущности
    /// </summary>
    public class DrawingBulldozer
    {
        /// <summary>
        ///  Класс-сущность
        /// </summary>
        public EntityBulldozer? EntityBulldozer { get; private set; }
        /// <summary>
        ///  Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        ///  Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        ///  Координата прорисовки по оси X
        /// </summary>
        private int _startPosX;
        /// <summary>
        ///  Координата прорисовки по оси Y
        /// </summary>
        private int _startPosY;
        /// <summary>
        ///  Ширина бульдозера
        /// </summary>
        private readonly int _bulldozerWidth = 150;
        /// <summary>
        ///  Высота бульдозера
        /// </summary>
        private readonly int _bulldozerHeight = 60;

        /// <summary>
        ///  
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес бульдозера</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="blade">Отвал бульдозера</param>
        /// <param name="ripper">Рыхлитель бульдозера</param>
        /// <param name="width">Ширина изображения</param>
        /// <param name="height">Высота изображения</param>

        public bool Init(int speed, double weight, Color bodyColor, Color additionalColor, Color dopColor, bool blade, bool ripper, int width, int height)
        {
            if ((height < _bulldozerHeight) || (width < _bulldozerWidth))
            {
                return false;
            }
            else
            {
                _pictureWidth = width;
                _pictureHeight = height;
                EntityBulldozer = new EntityBulldozer();
                EntityBulldozer.Init(speed, weight, bodyColor, additionalColor, dopColor, blade, ripper);
                return true;
            }
        }

        public void SetPosition(int x, int y)
        {
            if (x < 0)
            {
                x = 0;
            }
            else if (x > _pictureWidth - _bulldozerWidth)
            {
                x = _pictureWidth - _bulldozerWidth;
            }

            if (y < 0)
            {
                y = 0;
            }
            else if (y > _pictureHeight - _bulldozerHeight)
            {
                y = _pictureHeight - _bulldozerHeight;
            }
            _startPosX = x;
            _startPosY = y;
        }

        public void MoveBulldozer(DirectionType direction)
        {
            if (EntityBulldozer == null)
            {
            return;
            } 
            switch (direction)
            {
                case DirectionType.Left:
                    if (_startPosX - EntityBulldozer.Step > 0)
                    {
                        _startPosX -= (int)EntityBulldozer.Step;
                    }
                    break;
                case DirectionType.Up:
                    if (_startPosY - EntityBulldozer.Step > 0)
                    {
                        _startPosY -= (int)EntityBulldozer.Step;
                    }
                    break;
                case DirectionType.Right:
                    if (_startPosX + _bulldozerWidth + EntityBulldozer.Step < _pictureWidth)
                    {
                        _startPosX += (int)EntityBulldozer.Step;
                    }
                    break;
                case DirectionType.Down:
                    if (_startPosY + _bulldozerHeight + EntityBulldozer.Step < _pictureHeight)
                    {
                        _startPosY += (int)EntityBulldozer.Step;
                    }
                    break;
            }
        }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntityBulldozer == null)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush bodyBrush = new SolidBrush(EntityBulldozer.BodyColor);
        Brush additionalBrush = new SolidBrush(EntityBulldozer.AdditionalColor);
        Brush dopBrush = new SolidBrush(EntityBulldozer.DopColor);
        Pen dopPen = new Pen(EntityBulldozer.DopColor);

            /// <summary>
            /// <summary>
            /// Отрисовка отвала бульдозера
            /// </summary>
            if (EntityBulldozer.Blade) 
            {
                Point point1 = new Point(_startPosX + 8, _startPosY + 29);
                Point point2 = new Point(_startPosX + 8, _startPosY + 50);
                Point point3 = new Point(_startPosX, _startPosY + 50);
                Point[] triangle = { point1, point2, point3 };

                g.FillRectangle(dopBrush, _startPosX + 8, _startPosY + 29, 8, 8); // заливка основания отвала
                g.FillPolygon(dopBrush, triangle); // заливка отвала

                g.DrawRectangle(dopPen, _startPosX+8, _startPosY + 29, 8, 8); // обводка основания отвала
                g.DrawPolygon(dopPen, triangle); // обводка отвала
            }
            /// <summary>
            /// Отрисовка рыхлителя бульдозера
            /// </summary>
            if (EntityBulldozer.Ripper)
            {
                Point ripperPoint1 = new Point(_startPosX + 140, _startPosY + 37);
                Point ripperPoint2 = new Point(_startPosX + 151, _startPosY + 37);
                Point ripperPoint3 = new Point(_startPosX + 140, _startPosY + 60);
                Point[] ripper = { ripperPoint1, ripperPoint2, ripperPoint3 };

                g.FillRectangle(dopBrush, _startPosX + 136, _startPosY + 29, 15, 8); // заливка основания рыхлителя
                g.FillPolygon(dopBrush, ripper); // заливка отвала

                g.DrawRectangle(dopPen, _startPosX + 136, _startPosY + 29, 15, 8); // обводка основания рыхлителя
                g.DrawPolygon(dopPen, ripper);
            }

            /// <summary>
            /// Заливка гусеницы бульдозера 
            /// </summary>
            Brush brGray = new SolidBrush(Color.Gray);

            g.FillEllipse(brGray, _startPosX + 17, _startPosY + 24, 119, 40); //Гусеница
            g.FillEllipse(additionalBrush, _startPosX + 20, _startPosY + 35, 20, 20); // Левое колесо гусеницы
            g.FillEllipse(additionalBrush, _startPosX + 115, _startPosY + 35, 20, 20); // Правое колесо гусеницы
            g.FillEllipse(additionalBrush, _startPosX + 50, _startPosY + 45, 10, 10); // 1 центральное колесо гусеницы
            g.FillEllipse(additionalBrush, _startPosX + 70, _startPosY + 45, 10, 10); // 2 центральное колесо гусеницы
            g.FillEllipse(additionalBrush, _startPosX + 90, _startPosY + 45, 10, 10); // 3 центральное колесо гусеницы

            /// <summary>
            /// Отрисовка границ бульдозера
            /// </summary> 

            g.DrawEllipse(pen, _startPosX + 20, _startPosY + 35, 20, 20); // Левое колесо гусеницы
            g.DrawEllipse(pen, _startPosX + 115, _startPosY + 35, 20, 20); // Правое колесо гусеницы
            g.DrawEllipse(pen, _startPosX + 50, _startPosY + 45, 10, 10); // 1 центральное колесо гусеницы
            g.DrawEllipse(pen, _startPosX + 70, _startPosY + 45, 10, 10); // 2 центральное колесо гусеницы
            g.DrawEllipse(pen, _startPosX + 90, _startPosY + 45, 10, 10); // 3 центральное колесо гусеницы


            /// <summary>
            /// Кузов бульдозера
            /// </summary>
            g.FillRectangle(bodyBrush, _startPosX + 102, _startPosY, 28, 24); //кабина
            g.FillRectangle(bodyBrush, _startPosX + 17, _startPosY + 24, 119, 18); // основная часть
            g.FillRectangle(bodyBrush, _startPosX + 30, _startPosY, 10, 24); // выхлопная труба

            /// <summary>
            /// Корпус
            /// </summary>
            g.DrawRectangle(pen, _startPosX + 102, _startPosY, 28, 24); //кабина
            g.DrawRectangle(pen, _startPosX + 17, _startPosY + 24, 119, 18); // основная часть
            g.DrawRectangle(pen, _startPosX + 30, _startPosY, 10, 24); // выхлопная труба

            
            
        }
    }
}
